//
//  StatisticsViewController.swift
//  PER
//
//  Created by Conrado on 21/02/2020.
//  Copyright © 2020 Conrado. All rights reserved.
//

import UIKit
import MSBBarChart

private let reuseIdentifier = "DropDownCell"

enum Chapters: Int, CaseIterable {
    
    case RealExam
    case Chapter1
    case Chapter2
    case Chapter3
    case Chapter4
    case Chapter5
    case Chapter6
    case Chapter7
    case Chapter8
    case Chapter9
    case Chapter10
    case Chapter11
    
    var description : String {
        switch self {
        case .RealExam: return "Simulación de examen"
        case .Chapter1: return "Tema 1: Nomenclatura náutica"
        case .Chapter2: return "Tema 2: Amarre y fondeo"
        case .Chapter3: return "Tema 3: Seguridad en el mar"
        case .Chapter4: return "Tema 4: Legislación"
        case .Chapter5: return "Tema 5: Balizamiento"
        case .Chapter6: return "Tema 6: Reglamento RIPA"
        case .Chapter7: return "Tema 7: Maniobra"
        case .Chapter8: return "Tema 8: Emergencias en la mar"
        case .Chapter9: return "Tema 9: Meteorología"
        case .Chapter10: return "Tema 10: Teoría de navegación"
        case .Chapter11: return "Tema 11: Carta náutica"
        }
    }
    
}



class StatisticsViewController: UIViewController {
    
    
    @IBOutlet weak var menuButton: UIBarButtonItem!
    @IBOutlet weak var viewPresentation: UIView!
    var tableView : UITableView!
    @IBOutlet weak var barChartView: MSBBarChartView!
    @IBOutlet weak var viewHistory: UIView!
    @IBOutlet weak var buttonHistory: UIButton!
    @IBOutlet weak var imageHistory: UIImageView!
    
    
    // Tamaño de pantalla
    var realHeightScreen : CGFloat!
    var realWidthScreen : CGFloat!
    var initialPoint : CGFloat!
    var endPoint : CGFloat!
    var statusBarHeight : CGFloat!
    var navBarHeight : CGFloat!
    var tabBarHeight : CGFloat!
    
    var imageHeader : UIImage = UIImage(systemName: "arrowtriangle.down.fill")!
    var imageViewHeader : UIImageView!
    
    var showMenu : Bool = false
    var isExpanded : Bool = true
    var isSelected : Bool = false
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        InitialAndEndPoint ()
 
        // Se asigna el diseño de la barra de navegación
        setDesignNavBar(navigationItem: navigationItem, appearance: UINavigationBarAppearance())
 
        configureTableView()
        
        menuButton.target = self.revealViewController()
        menuButton.action = #selector(SWRevealViewController.revealToggle(_:))
        self.view.addGestureRecognizer(self.revealViewController().panGestureRecognizer())
        self.revealViewController().rearViewRevealWidth = fullWidthScreen * 0.85
        
        self.barChartView.backgroundColor = .clear
        
        // - - - - -
        
        barChartView.frame = CGRect (
            x: self.realWidthScreen*0.02,
            y: initialPoint + 5,
            width: fullWidthScreen - self.realWidthScreen*0.04,
            height: realHeightScreen - 10
        )
        
        viewHistory.translatesAutoresizingMaskIntoConstraints = false
        buttonHistory.translatesAutoresizingMaskIntoConstraints = false
        imageHistory.translatesAutoresizingMaskIntoConstraints = false
        
        viewHistory.topAnchor.constraint(equalTo: viewPresentation.bottomAnchor, constant: 8).isActive = true
        viewHistory.leftAnchor.constraint(equalTo: view.leftAnchor, constant: self.realWidthScreen*0.62).isActive = true
        viewHistory.rightAnchor.constraint(equalTo: view.rightAnchor, constant: -self.realWidthScreen*0.02).isActive = true
        viewHistory.heightAnchor.constraint(equalToConstant: 45).isActive = true
        
        buttonHistory.topAnchor.constraint(equalTo: viewHistory.topAnchor).isActive = true
        buttonHistory.bottomAnchor.constraint(equalTo: viewHistory.bottomAnchor).isActive = true
        buttonHistory.leftAnchor.constraint(equalTo: viewHistory.leftAnchor).isActive = true
        buttonHistory.rightAnchor.constraint(equalTo: viewHistory.rightAnchor).isActive = true
        
        imageHistory.heightAnchor.constraint(equalToConstant: 22.5).isActive = true
        imageHistory.widthAnchor.constraint(equalToConstant: 22.5).isActive = true
        imageHistory.rightAnchor.constraint(equalTo: viewHistory.rightAnchor, constant: -10).isActive = true
        imageHistory.centerYAnchor.constraint(equalTo: viewHistory.centerYAnchor).isActive = true

        viewHistory.clipsToBounds = true
        viewHistory.layer.cornerRadius = 10
        // viewHistory.layer.borderColor = mainBlue.cgColor
        // viewHistory.layer.borderWidth = 1
        
        imageHistory.tintColor = mainBlue
        buttonHistory.tintColor = mainBlue
        
        viewHistory.isHidden = true
        
        
    }
    
    
    @objc func handleDropDown () {
        
        tableView.separatorStyle = .singleLine
        tableView.separatorColor = secondaryBlueColor
        self.tableView.separatorInset = UIEdgeInsets(top: 0, left: 30, bottom: 0, right: 30)

        
        if (imageHeader == UIImage(systemName: "arrowtriangle.up.fill")) {
            imageHeader = UIImage(systemName: "arrowtriangle.down.fill")!
            imageViewHeader.image = imageHeader
        } else if (imageHeader == UIImage(systemName: "arrowtriangle.down.fill")) {
            imageHeader = UIImage(systemName: "arrowtriangle.up.fill")!
            imageViewHeader.image = imageHeader
        }
        
        if (isExpanded == false) {
            print ("Estoy reducido. Tengo que ampliarme ... ")
            UIView.animate(withDuration: 2, animations: {
                self.tableView.widthAnchor.constraint(equalToConstant: 500).isActive = true
                self.view.layoutIfNeeded()
            })
            isExpanded = true
        }
        
        showMenu = !showMenu
                
        var indexPaths = [IndexPath] ()
        
        Chapters.allCases.forEach { (chapter) in
            let indexPath = IndexPath(row: chapter.rawValue, section: 0)
            indexPaths.append(indexPath)
        }
        
        if showMenu {
            tableView.separatorStyle = .singleLine
            tableView.insertRows(at: indexPaths, with: .fade)
        } else {
            tableView.separatorStyle = .none
            tableView.deleteRows(at: indexPaths, with: .fade)
        }
        
    }
    
    
    func configureTableView () {
        tableView = UITableView ()
        tableView.delegate = self
        tableView.dataSource = self
        tableView.translatesAutoresizingMaskIntoConstraints = false
        tableView.separatorStyle = .none
        tableView.separatorColor = UIColor.clear
        // tableView.isScrollEnabled = false
        // tableView.bounces = true
        tableView.rowHeight = 40
        tableView.clipsToBounds = true
        tableView.layer.cornerRadius = 10
        tableView.register(DropDownMenuCell.self, forCellReuseIdentifier: reuseIdentifier)
        tableView.backgroundColor = UIColor.clear
        view.addSubview(tableView)
        tableView.topAnchor.constraint(equalTo: viewPresentation.bottomAnchor, constant: 8).isActive = true
        tableView.leftAnchor.constraint(equalTo: view.leftAnchor, constant: self.realWidthScreen*0.02).isActive = true
        tableView.rightAnchor.constraint(equalTo: view.rightAnchor, constant: -self.realWidthScreen*0.02).isActive = true
        tableView.heightAnchor.constraint(equalToConstant: 260).isActive = true
    }
    
    func InitialAndEndPoint () {
        // height of common elements: statusBar, navBar and tabBar
        statusBarHeight = getStatusBarHeight()
        navBarHeight = getNavBarHeight(navigationController: self.navigationController!)
        tabBarHeight = getTabBarHeight(tabBarController: self.tabBarController!)
        // real height and real width of screen
        realHeightScreen = fullHeightScreen - statusBarHeight - navBarHeight - tabBarHeight - viewPresentation.frame.size.height
        realHeightScreen = realHeightScreen - 53
        realWidthScreen = fullWidthScreen
        // initial point and end point of screen
        initialPoint = getTopBarHeight(statusBarHeight: statusBarHeight, navBarHeight: navBarHeight) + viewPresentation.frame.size.height + 53
        endPoint = fullHeightScreen - tabBarHeight
    }
    
    
    
    
}




extension StatisticsViewController : UITableViewDelegate, UITableViewDataSource {
    
    // Altura de la HEADER SECTION
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return 40
    }
    
    // Vista de la HEADER SECTION
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        
        let viewHeader : UIView = UIView ()
        let buttonHeader : UIButton = UIButton(type: .system)
        
        buttonHeader.setTitle("Selecciona uno de los temas", for: .normal)
        buttonHeader.setTitleColor(.white, for: .normal)
        buttonHeader.titleLabel?.font = UIFont(name: "IBMPlexSans-Bold", size: 16)
        buttonHeader.addTarget(self, action: #selector(handleDropDown), for: .touchUpInside)
        buttonHeader.backgroundColor = secondaryBlueColor
        
        imageViewHeader = UIImageView(image: imageHeader)
        imageViewHeader.tintColor = .white
        
        viewHeader.addSubview(buttonHeader)
        viewHeader.addSubview(imageViewHeader)
        
        buttonHeader.translatesAutoresizingMaskIntoConstraints = false
        buttonHeader.centerXAnchor.constraint(equalTo: viewHeader.centerXAnchor).isActive = true
        buttonHeader.centerYAnchor.constraint(equalTo: viewHeader.centerYAnchor).isActive = true
        buttonHeader.widthAnchor.constraint(equalTo: viewHeader.widthAnchor).isActive = true
        buttonHeader.heightAnchor.constraint(equalTo: viewHeader.heightAnchor).isActive = true
        
        imageViewHeader.translatesAutoresizingMaskIntoConstraints = false
        imageViewHeader.centerYAnchor.constraint(equalTo: viewHeader.centerYAnchor).isActive = true
        imageViewHeader.widthAnchor.constraint(equalToConstant: 15).isActive = true
        imageViewHeader.heightAnchor.constraint(equalToConstant: 15).isActive = true
        imageViewHeader.rightAnchor.constraint(equalTo: viewHeader.rightAnchor, constant: -10).isActive = true
        
        return viewHeader
        
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return showMenu ? Chapters.allCases.count : 0
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: reuseIdentifier, for: indexPath) as! DropDownMenuCell
        cell.titleLabel.text = Chapters(rawValue: indexPath.row)?.description
        cell.titleLabel.textAlignment = .center
        cell.backgroundColor = UIColor(red: 255.0/255.0, green: 229.0/255.0, blue: 165.0/255.0, alpha: 1.0)
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        
        showMenu = !showMenu
                
        var indexPaths = [IndexPath] ()
        
        Chapters.allCases.forEach { (chapter) in
            let indexPath = IndexPath(row: chapter.rawValue, section: 0)
            indexPaths.append(indexPath)
        }
        
        if showMenu {
            tableView.separatorStyle = .singleLine
            tableView.insertRows(at: indexPaths, with: .fade)
        } else {
            tableView.separatorStyle = .none
            tableView.deleteRows(at: indexPaths, with: .fade)
        }
        
        if isExpanded {
            UIView.animate(withDuration: 2, delay: 0.5, options: .curveEaseOut, animations: {
                tableView.rightAnchor.constraint(equalTo: self.view.rightAnchor, constant: -self.realWidthScreen*0.43).isActive = true
                self.view.layoutIfNeeded()
            })
            isExpanded = false
        }
        
        viewHistory.isHidden = false
        
        self.barChartView.setOptions([.yAxisTitle("Exámenes"), .yAxisNumberOfInterval(10)])
        self.barChartView.assignmentOfColor = [3.9..<4.1: .red, 4.9..<5.1: .green, 8.9..<9.1: .blue]
        self.barChartView.setDataEntries(values: [4, 5, 9])
        self.barChartView.setXAxisUnitTitles(["Realizados", "Aprobados", "Suspensos"])
        self.barChartView.start()
        self.barChartView.prueba()
        
        // print ("El usuario está pulsando sobre la celda de \((Chapters(rawValue: indexPath.row)?.description)!)")
        // print ("El usuario está pulsando sobre la celda número \(indexPath.row)")
        
    }
    
}

extension MSBBarChartView {
    func prueba () {
        print (" ... --- ... --- ... --- ... --- ... --- ... ")
        
    }
}
