//
//  StatisticsViewController.swift
//  PER
//
//  Created by Conrado on 21/02/2020.
//  Copyright © 2020 Conrado. All rights reserved.
//

import UIKit
import MSBBarChart

private let reuseIdentifier = "DropDownCell"

enum Chapters: Int, CaseIterable {
    
    case RealExam
    case Chapter1
    case Chapter2
    case Chapter3
    case Chapter4
    case Chapter5
    case Chapter6
    case Chapter7
    case Chapter8
    case Chapter9
    case Chapter10
    case Chapter11
    
    var description : String {
        switch self {
        case .RealExam: return "Simulación de examen"
        case .Chapter1: return "Tema 1: Nomenclatura náutica"
        case .Chapter2: return "Tema 2: Amarre y fondeo"
        case .Chapter3: return "Tema 3: Seguridad en el mar"
        case .Chapter4: return "Tema 4: Legislación"
        case .Chapter5: return "Tema 5: Balizamiento"
        case .Chapter6: return "Tema 6: Reglamento RIPA"
        case .Chapter7: return "Tema 7: Maniobra"
        case .Chapter8: return "Tema 8: Emergencias en la mar"
        case .Chapter9: return "Tema 9: Meteorología"
        case .Chapter10: return "Tema 10: Teoría de navegación"
        case .Chapter11: return "Tema 11: Carta náutica"
        }
    }
}



class StatisticsViewController: UIViewController {
    
    @IBOutlet weak var menuButton: UIBarButtonItem!
    @IBOutlet weak var viewPresentation: UIView!
    var tableView : UITableView!
    @IBOutlet weak var barChartView: MSBBarChartView!
    
    
    // Tamaño de pantalla
    var realHeightScreen : CGFloat!
    var realWidthScreen : CGFloat!
    var initialPoint : CGFloat!
    var endPoint : CGFloat!
    var statusBarHeight : CGFloat!
    var navBarHeight : CGFloat!
    var tabBarHeight : CGFloat!
    
    // - - - - - - -
    var imageHeader : UIImage = UIImage(systemName: "arrowtriangle.down.fill")!
    var imageViewHeader : UIImageView!
    var showMenu : Bool = false
    var isExpanded : Bool = true
    var isSelected : Bool = false
    var counter = 0
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        InitialAndEndPoint ()
         
        // Se asigna el diseño de la barra de navegación
        setDesignNavBar(navigationItem: navigationItem, appearance: UINavigationBarAppearance())
 
        configureTableView()
        
        menuButton.target = self.revealViewController()
        menuButton.action = #selector(SWRevealViewController.revealToggle(_:))
        self.view.addGestureRecognizer(self.revealViewController().panGestureRecognizer())
        self.revealViewController().rearViewRevealWidth = fullWidthScreen * 0.85
        
        self.barChartView.backgroundColor = .clear
        
        // - - - - -
        
        viewPresentation.clipsToBounds = true
        viewPresentation.layer.cornerRadius = 27
        viewPresentation.layer.maskedCorners = [.layerMaxXMaxYCorner, .layerMinXMaxYCorner]
        
        // - - - - -
        
        barChartView.frame = CGRect (
            x: self.realWidthScreen*0.02,
            y: initialPoint + 5,
            width: fullWidthScreen - self.realWidthScreen*0.04,
            height: realHeightScreen - 10
        )
    }
    
    override func viewDidAppear(_ animated: Bool) {
        if counter == 0 {
            reinitGraph ()
            counter = 1
        }
    }
    
    func reinitGraph () {
        // Gráfico
        self.barChartView.setOptions([.yAxisTitle("Exámenes"), .yAxisNumberOfInterval(10)])
        self.barChartView.assignmentOfColor = [0.0..<0.5: .red, 0.5..<0.75: .green, 0.75..<1.0: .blue]
        self.barChartView.setDataEntries(values: [0, 0, 0])
        self.barChartView.setXAxisUnitTitles(["Realizados", "Aprobados", "Suspensos"])
        self.barChartView.start()
    }
    
    @objc func handleDropDown () {
        
        showMenu = !showMenu
        
        tableView.separatorStyle = .none
        tableView.separatorColor = secondaryBlueColor
        self.tableView.separatorInset = UIEdgeInsets(top: 0, left: 15, bottom: 0, right: 15)

        if (imageHeader == UIImage(systemName: "arrowtriangle.up.fill")) {
            imageHeader = UIImage(systemName: "arrowtriangle.down.fill")!
            imageViewHeader.image = imageHeader
        } else if (imageHeader == UIImage(systemName: "arrowtriangle.down.fill")) {
            imageHeader = UIImage(systemName: "arrowtriangle.up.fill")!
            imageViewHeader.image = imageHeader
        }
        
        var indexPaths = [IndexPath] ()
        
        Chapters.allCases.forEach { (chapter) in
            let indexPath = IndexPath(row: chapter.rawValue, section: 0)
            indexPaths.append(indexPath)
        }
        
        if showMenu {
            tableView.separatorStyle = .singleLine
            tableView.insertRows(at: indexPaths, with: .fade)
            tableView.backgroundColor = backgroundColor
        } else {
            tableView.separatorStyle = .none
            tableView.deleteRows(at: indexPaths, with: .fade)
            tableView.backgroundColor = UIColor.clear
        }
        
    }
    
    
    func configureTableView () {
        tableView = UITableView ()
        tableView.delegate = self
        tableView.dataSource = self
        tableView.translatesAutoresizingMaskIntoConstraints = false
        tableView.separatorStyle = .none
        tableView.separatorColor = UIColor.clear
        // tableView.isScrollEnabled = false
        tableView.bounces = false
        tableView.rowHeight = 40
        tableView.clipsToBounds = true
        tableView.layer.cornerRadius = 10
        tableView.register(DropDownMenuCell.self, forCellReuseIdentifier: reuseIdentifier)
        tableView.backgroundColor = UIColor.clear
        view.addSubview(tableView)
        tableView.topAnchor.constraint(equalTo: viewPresentation.bottomAnchor, constant: 8).isActive = true
        tableView.leftAnchor.constraint(equalTo: view.leftAnchor, constant: self.realWidthScreen*0.02).isActive = true
        tableView.rightAnchor.constraint(equalTo: view.rightAnchor, constant: -self.realWidthScreen*0.02).isActive = true
        tableView.heightAnchor.constraint(equalToConstant: 260).isActive = true
    }
    
    func InitialAndEndPoint () {
        // height of common elements: statusBar, navBar and tabBar
        statusBarHeight = getStatusBarHeight()
        navBarHeight = getNavBarHeight(navigationController: self.navigationController!)
        tabBarHeight = getTabBarHeight(tabBarController: self.tabBarController!)
        // real height and real width of screen
        realHeightScreen = fullHeightScreen - statusBarHeight - navBarHeight - tabBarHeight - viewPresentation.frame.size.height
        realHeightScreen = realHeightScreen - 53
        realWidthScreen = fullWidthScreen
        // initial point and end point of screen
        initialPoint = getTopBarHeight(statusBarHeight: statusBarHeight, navBarHeight: navBarHeight) + viewPresentation.frame.size.height + 53
        endPoint = fullHeightScreen - tabBarHeight
    }
    
    // Falta por reiniciar el gráfico una vez eliminado
    @IBAction func resetStatistics(_ sender: UIBarButtonItem) {
        let alert = UIAlertController(title: "Reiniciar estadísticas", message: "\n ¿Seguro que quieres reiniciar las estadísticas? No podrás recuperarlas una vez borradas...", preferredStyle: .alert)
        let actionThisChapter = UIAlertAction(title: "Sí, del tema seleccionado", style: .default, handler: { action in
            if self.buttonHeader.titleLabel?.text == "Selecciona uno de los temas" {
                let alert_2 = UIAlertController(title: "Selecciona un tema", message: "\n No ha sido posible reiniciar las estadísticas ya que no has escogido ninguno de los temas", preferredStyle: .alert)
                let actionOk = UIAlertAction(title: "OK", style: .default, handler: nil)
                alert_2.addAction(actionOk)
                self.present(alert_2, animated: true, completion: nil)
            } else {
                removeHistoryOfChapter(chapter: self.chapterSelected)
                let alert_3 = feedBackRemoveData(chapter: "Todos los exámenes del tema \(self.chapterSelected) han sido eliminados")
                self.present(alert_3, animated: true, completion: nil)
                self.reinitGraph ()
            }
        })
        let actionAllChapters = UIAlertAction(title: "Sí, de todos los temas", style: .default, handler: { action in
            removeAllHistory ()
            let alert_3 = feedBackRemoveData(chapter: "Todos los exámenes han sido eliminados")
            self.present(alert_3, animated: true, completion: nil)
            self.reinitGraph ()
        })
        let actionCancel = UIAlertAction(title: "Cancelar", style: .destructive, handler: nil)
        alert.addAction(actionThisChapter)
        alert.addAction(actionAllChapters)
        alert.addAction(actionCancel)
        present(alert, animated: true, completion: nil)
    }
    
    
    
    
    
    let buttonHeader : UIButton = UIButton (type: .system)
    var chapterSelected : String = ""
}




extension StatisticsViewController : UITableViewDelegate, UITableViewDataSource {
    
    // Altura de la HEADER SECTION
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return 40
    }
    
    // Vista de la HEADER SECTION
    
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        // let buttonHeader : UIButton = UIButton (type: .system)
        let viewHeader : UIView = UIView ()

        buttonHeader.setTitle("Selecciona uno de los temas", for: .normal)
        buttonHeader.setTitleColor(.white, for: .normal)
        buttonHeader.titleLabel?.font = UIFont(name: "IBMPlexSans-Bold", size: 16)
        buttonHeader.addTarget(self, action: #selector(handleDropDown), for: .touchUpInside)
        buttonHeader.backgroundColor = secondaryBlueColor
        
        imageViewHeader = UIImageView(image: imageHeader)
        imageViewHeader.tintColor = .white
        
        viewHeader.addSubview(buttonHeader)
        viewHeader.addSubview(imageViewHeader)
        
        buttonHeader.translatesAutoresizingMaskIntoConstraints = false
        buttonHeader.centerXAnchor.constraint(equalTo: viewHeader.centerXAnchor).isActive = true
        buttonHeader.centerYAnchor.constraint(equalTo: viewHeader.centerYAnchor).isActive = true
        buttonHeader.widthAnchor.constraint(equalTo: viewHeader.widthAnchor).isActive = true
        buttonHeader.heightAnchor.constraint(equalTo: viewHeader.heightAnchor).isActive = true
        
        imageViewHeader.translatesAutoresizingMaskIntoConstraints = false
        imageViewHeader.centerYAnchor.constraint(equalTo: viewHeader.centerYAnchor).isActive = true
        imageViewHeader.widthAnchor.constraint(equalToConstant: 15).isActive = true
        imageViewHeader.heightAnchor.constraint(equalToConstant: 15).isActive = true
        imageViewHeader.rightAnchor.constraint(equalTo: viewHeader.rightAnchor, constant: -15).isActive = true
        
        buttonHeader.clipsToBounds = true
        buttonHeader.layer.cornerRadius = 10
        viewHeader.backgroundColor = backgroundColor
        
        return viewHeader
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return showMenu ? Chapters.allCases.count : 0
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: reuseIdentifier, for: indexPath) as! DropDownMenuCell
        cell.titleLabel.text = Chapters(rawValue: indexPath.row)?.description
        cell.titleLabel.textAlignment = .center
        cell.backgroundColor = UIColor.white
        cell.layer.cornerRadius = 10
        return cell
    }
    
    func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        let result = UIView()
        let insets = tableView.separatorInset
        let width = tableView.bounds.width - insets.left - insets.right
        let sepFrame = CGRect(x: insets.left, y: -0.5, width: width, height: 0.5)
        let sep = CALayer()
        sep.frame = sepFrame
        sep.backgroundColor = tableView.separatorColor?.cgColor
        result.layer.addSublayer(sep)
        return result
    }
    
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        
        if (imageHeader == UIImage(systemName: "arrowtriangle.up.fill")) {
            imageHeader = UIImage(systemName: "arrowtriangle.down.fill")!
            imageViewHeader.image = imageHeader
        } else if (imageHeader == UIImage(systemName: "arrowtriangle.down.fill")) {
            imageHeader = UIImage(systemName: "arrowtriangle.up.fill")!
            imageViewHeader.image = imageHeader
        }
        
        showMenu = !showMenu
        
        if indexPath.row == 0 {
            buttonHeader.setTitle(optionExamName[indexPath.row], for: .normal)
        } else {
            buttonHeader.setTitle(fullChapters[indexPath.row-1], for: .normal)
        }
        
        var indexPaths = [IndexPath] ()
        
        Chapters.allCases.forEach { (chapter) in
            let indexPath = IndexPath(row: chapter.rawValue, section: 0)
            indexPaths.append(indexPath)
        }
        
        if showMenu {
            tableView.separatorStyle = .singleLine
            tableView.insertRows(at: indexPaths, with: .fade)
            tableView.backgroundColor = backgroundColor
        } else {
            tableView.separatorStyle = .none
            tableView.deleteRows(at: indexPaths, with: .fade)
            tableView.backgroundColor = UIColor.clear
        }
        
        // =============
        // =============
        // =============
        var tests : [Test] = []
        
        if indexPath.row == 0 {  chapterSelected = "Simulación de examen real"  }
        else { chapterSelected = chaptersName[indexPath.row-1] }
        
        print ("Estoy consiguiendo los exámenes de \(chapterSelected)")
        
        do {
            for row in try databaseG.prepare("SELECT * FROM Exámenes WHERE temario = '\(chapterSelected)'") {
                // - - - - - -
                let id : Int = Int(Optional(row[0]) as! Int64)
                let chapter : String? = row[1] as? String
                let numb_questions : Int = Int(Optional(row[2]) as! Int64)
                let corr_questions : Int = Int(Optional(row[3]) as! Int64)
                let wron_questions : Int = Int(Optional(row[4]) as! Int64)
                let appr : Int = Int(Optional(row[5]) as! Int64)
                // - - - - - -
                let testToInsert : Test!
                if (appr == 0) {
                    testToInsert = Test(id: id, chapter: chapter!, questions: numb_questions, correctQ: corr_questions, wrongQ: wron_questions, approved: false)
                } else {
                    testToInsert = Test(id: id, chapter: chapter!, questions: numb_questions, correctQ: corr_questions, wrongQ: wron_questions, approved: true)
                }
                // - - - - - -
                tests.append(testToInsert)
            }
        } catch {   print (error)   }
        
        print (" - - - - El count del array de TESTS realizados es: \(tests.count)")
        // print ("id: \(tests[0].idExam)" + "\n" + "questions: \(tests[0].numberQuestions)")
        // print ("id: \(tests[1].idExam)" + "\n" + "questions: \(tests[1].numberQuestions)")
        
        var approvedExams : Int = 0
        var suspendedExams : Int = 0
        
        if tests.isEmpty == false {
            for i in 0...tests.count-1 {
                if tests[i].approved == true {
                    approvedExams = approvedExams + 1
                } else {
                    suspendedExams = suspendedExams + 1
                }
            }
        }
        // =============
        // =============
        // =============
        
        self.barChartView.setOptions([.yAxisTitle("Exámenes"), .yAxisNumberOfInterval(10)])
        self.barChartView.assignmentOfColor = [0.0..<10.0: secondaryBlueColor]
        self.barChartView.setDataEntries(values: [tests.count, approvedExams, suspendedExams])
        self.barChartView.setXAxisUnitTitles(["Realizados", "Aprobados", "Suspensos"])
        self.barChartView.start()
        
        // print ("El usuario está pulsando sobre la celda de \((Chapters(rawValue: indexPath.row)?.description)!)")
        // print ("El usuario está pulsando sobre la celda número \(indexPath.row)")
        
    }
    
    
    
    
    
    
    
    
    
    
}


