//
//  ChapterController.swift
//  PER
//
//  Created by Conrado on 24/01/2020.
//  Copyright © 2020 Conrado. All rights reserved.
//

import UIKit

class ChapterController : UIViewController, UICollectionViewDelegate, UICollectionViewDataSource, UICollectionViewDelegateFlowLayout {

    /* Etiquetas de los elementos de user interface */
    /* ++++++++++++++++++++++++++++++++++++++++++++ */
    @IBOutlet weak var sideMenuButton: UIBarButtonItem!
    @IBOutlet weak var collectionView : UICollectionView!
    @IBOutlet weak var viewPresentationNavBar: UIView!
    @IBOutlet weak var titlePresentationNavBar: UILabel!
    @IBOutlet weak var segmContPresentationNavBar: HBSegmentedControl!
    
    /* Variables para la definición de tamaños de pantalla */
    /* +++++++++++++++++++++++++++++++++++++++++++++++++++ */
    var realHeightScreen : CGFloat!
    var realWidthScreen : CGFloat!
    var initialPoint : CGFloat!
    var endPoint : CGFloat!
    var statusBarHeight : CGFloat!
    var navBarHeight : CGFloat!
    var tabBarHeight : CGFloat!
    
    /* Otras variables de control */
    /* ++++++++++++++++++++++++++ */
    var marginsCollectionView : CGFloat = 0.0
    var nameChapter : [String] = []
    var imageChapter : [String] = []
    var selectedCell : Int!
    

    
    // MARK: - viewDidLoad() y viewWillAppear()
    
    override func viewDidLoad () {
        super.viewDidLoad()
    
        // Determinación del tamaño de pantalla
        InitialAndEndPoint ()
        
        // Asignación de datos iniciales
        segmContPresentationNavBar.items = ["Examen organizado por temarios", "Simulación de examen real"]
        segmContPresentationNavBar.selectedIndex = 0
        segmContPresentationNavBar.addTarget(self, action: #selector(segmentValueChanged(_:)), for: .valueChanged)
        nameChapter = chaptersName
        imageChapter = optionChaptersImage
        
        // Posicionamiento del CollectionView
        marginsCollectionView = realWidthScreen * 0.035
        collectionView.frame = CGRect (x: marginsCollectionView, y: initialPoint + viewPresentationNavBar.frame.height + marginsCollectionView, width: realWidthScreen - (marginsCollectionView * 2), height: endPoint - initialPoint - viewPresentationNavBar.frame.height - (marginsCollectionView * 2))
        collectionView.layer.cornerRadius = 10
        
        // Delegación y proveedor de datos de collectionView
        collectionView.delegate = self
        collectionView.dataSource = self
        
        viewPresentationNavBar.clipsToBounds = true
        viewPresentationNavBar.layer.cornerRadius = 27
        viewPresentationNavBar.layer.maskedCorners = [.layerMaxXMaxYCorner, .layerMinXMaxYCorner]
        
        // Definición del menú lateral
        sideMenuButton.target = self.revealViewController()
        sideMenuButton.action = #selector(SWRevealViewController.revealToggle(_:))
        self.view.addGestureRecognizer(self.revealViewController().panGestureRecognizer())
        self.revealViewController().rearViewRevealWidth = realWidthScreen * 0.85

        // Se asigna el diseño de la barra de navegación
        setDesignNavBar (navigationItem: navigationItem, appearance: UINavigationBarAppearance())
        
        
    }
    
    override func viewWillAppear (_ animated: Bool) {
        super.viewWillAppear(animated)
        // Se activa siempre el tab bar de este viewController
        self.tabBarController?.tabBar.isHidden = false
        self.navigationController?.navigationBar.isHidden = false
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        // Se activa siempre el tab bar de este viewController
        self.tabBarController?.tabBar.isHidden = false
    }
    
    
    
    // MARK: - UICollectionView
    
    // Número de secciones en el Collection View
    func numberOfSections(in collectionView: UICollectionView) -> Int {
        return 1
    }
    
    // Número de celdas en la sección del Collection View
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return nameChapter.count
    }
    
    // Método cellForItemAtIndexPath
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "chapterCell", for: indexPath) as! ChapterCell

        if (segmContPresentationNavBar.selectedIndex == 0) {
            cell.numberChapterCell.text = "Tema \(indexPath.row + 1)"
        } else if (segmContPresentationNavBar.selectedIndex == 1) {
            cell.numberChapterCell.text = "Temario completo"
        }
        
        cell.titleChapterCell.text = nameChapter[indexPath.row]
        cell.imageChapterCell.image = UIImage(named: imageChapter[indexPath.row])
        
        cell.labelAccessExamChapterCell.clipsToBounds = true
        cell.labelAccessExamChapterCell.layer.cornerRadius = 7
        
        cell.layer.cornerRadius = 10
        cell.imageChapterCell.alpha = 0.8
            
        return cell
    }
        
    // Asignación de altura y anchura a cada celda
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        
        let width_cell : CGFloat = collectionView.bounds.width
        var height_cell : CGFloat = 0.0
        
        if (segmContPresentationNavBar.selectedIndex == 0) {
            height_cell = (width_cell/3) + 106
        } else if (segmContPresentationNavBar.selectedIndex == 1) {
            height_cell = collectionView.bounds.height
        }
        
        return CGSize(width: width_cell, height: height_cell)
    }
    
    // Asignación de margen superior e inferior entre líneas de Collection View
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, minimumLineSpacingForSectionAt section: Int) -> CGFloat {
        return marginsCollectionView
    }
    

    
    // MARK: - Acciones de los elementos de la user interface
    
    @objc func segmentValueChanged(_ sender: UISegmentedControl) {
        switch segmContPresentationNavBar.selectedIndex {
            case 0:
                nameChapter = chaptersName
                imageChapter = optionChaptersImage
                collectionView.reloadData()
            case 1:
                nameChapter = optionExamName
                imageChapter = optionExamImage
                collectionView.reloadData()
            default:
                break
        }
    }
    
    @IBAction func buttonAccessExam(_ button: UIButton) {
        if let indexPath = self.collectionView.indexPathForButtonCollectionView(button) {
            selectedCell = indexPath.row
        }
        
        let alert = UIAlertController(title: "Vas a acceder a un examen de \(nameChapter[selectedCell].lowercased())", message: "\n ¿Estás preparado? :-)", preferredStyle: .alert)
        let actionExit = UIAlertAction(title: "Cancelar", style: .destructive, handler: nil)
        let actionContinue = UIAlertAction(title: "¡Adelante!", style: .default, handler: { action in
            // SEGUE
            let sampleStoryBoard : UIStoryboard = UIStoryboard (name: "Main", bundle: nil)
            let testController = sampleStoryBoard.instantiateViewController(withIdentifier: "TestController") as! TestViewController
            testController.nameTest = self.nameChapter[self.selectedCell]
            self.navigationController?.pushViewController(testController, animated: true)
            self.dismiss(animated: true, completion: nil)
        })
        
        alert.addAction(actionExit)
        alert.addAction(actionContinue)
        present(alert, animated: true, completion: nil)
        
    }
    
    
    
    // MARK: - Método de obtención de los tamaños y puntos de la pantalla
    
    func InitialAndEndPoint () {
        statusBarHeight = getStatusBarHeight()
        navBarHeight = getNavBarHeight(navigationController: self.navigationController!)
        tabBarHeight = getTabBarHeight(tabBarController: self.tabBarController!)
        realHeightScreen = fullHeightScreen - statusBarHeight - navBarHeight - tabBarHeight
        realWidthScreen = fullWidthScreen
        initialPoint = getTopBarHeight(statusBarHeight: statusBarHeight, navBarHeight: navBarHeight)
        endPoint = fullHeightScreen - tabBarHeight
    }

}



// MARK: - Extensión del collectionView

extension UICollectionView {
    func indexPathForButtonCollectionView(_ view: UIView) -> IndexPath? {
        let center = view.center
        let viewCenter = self.convert(center, from: view.superview)
        let indexPath = self.indexPathForItem(at: viewCenter)
        return indexPath
    }
}





