//
//  IncidenceViewController.swift
//  PER
//
//  Created by Conrado on 22/04/2020.
//  Copyright © 2020 Conrado. All rights reserved.
//

import UIKit
import MessageUI

class IncidenceViewController: UIViewController, UITextViewDelegate, UITextFieldDelegate, MFMailComposeViewControllerDelegate {

    @IBOutlet weak var presentationView: UIView!
    @IBOutlet weak var explanationLabel: UILabel!
    @IBOutlet weak var subjectTextField: UITextField!
    @IBOutlet weak var explanationTextView: UITextView!
    @IBOutlet weak var sendView: UIView!
    @IBOutlet weak var sendImage: UIImageView!
    @IBOutlet weak var sendButton: UIButton!
    
    @IBOutlet weak var menuButton: UIBarButtonItem!
    
    // Tamaño de pantalla
    var realHeightScreen : CGFloat!
    var realWidthScreen : CGFloat!
    var initialPoint : CGFloat!
    var endPoint : CGFloat!
    var statusBarHeight : CGFloat!
    var navBarHeight : CGFloat!
    var tabBarHeight : CGFloat!
    
    var segueVC : String = ""
    
    let sugerencia_detail = "Escribe aquí el detalle de la sugerencia"
    let incidencia_detail = "Escribe aquí el detalle de la incidencia"
    var realOption_detail = ""
    let sugerencia_topic = "Escribe aquí el asunto de la sugerencia"
    let incidencia_topic = "Escribe aquí el asunto de la incidencia"
    var realOption_topic = ""
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        explanationTextView.delegate = self
        subjectTextField.delegate = self
        
        self.view.backgroundColor = backgroundColor
        
        presentationView.clipsToBounds = true
        presentationView.layer.cornerRadius = 27
        presentationView.layer.maskedCorners = [.layerMaxXMaxYCorner, .layerMinXMaxYCorner]
        
        menuButton.target = self.revealViewController()
        menuButton.action = #selector(SWRevealViewController.revealToggle(_:))
        self.view.addGestureRecognizer(self.revealViewController().panGestureRecognizer())
        self.revealViewController().rearViewRevealWidth = fullWidthScreen * 0.85
        
        // Se asigna el diseño de la barra de navegación
        setDesignNavBar (navigationItem: navigationItem, appearance: UINavigationBarAppearance())
        
        explanationTextView.textColor = UIColor(red: 169.0/255.0, green: 169.0/255.0, blue: 169.0/255.0, alpha: 1.0)
        subjectTextField.textColor = UIColor(red: 33.0/255.0, green: 33.0/255.0, blue: 33.0/255.0, alpha: 1.0)
        
        if (segueVC == "Incidencias") {
            realOption_topic = incidencia_topic
            realOption_detail = incidencia_detail
            explanationLabel.text = "Si has tenido alguna incidencia durante el uso de la aplicación, ¡cuéntanosla!" + "\n" + "Trataremos de solucionarla cuanto antes."
            subjectTextField.attributedPlaceholder = NSAttributedString(string: realOption_topic, attributes: [NSAttributedString.Key.foregroundColor: UIColor(red: 169.0/255.0, green: 169.0/255.0, blue: 169.0/255.0, alpha: 1.0)])
            explanationTextView.text = realOption_detail
        }
        
        if (segueVC == "Sugerencias") {
            realOption_topic = sugerencia_topic
            realOption_detail = sugerencia_detail
            explanationLabel.text = "Si has tenido alguna idea que crees que puede mejorar los servicios que ofrece la aplicación, ¡cuéntanosla!"
            subjectTextField.attributedPlaceholder = NSAttributedString(string: realOption_topic, attributes: [NSAttributedString.Key.foregroundColor: UIColor(red: 169.0/255.0, green: 169.0/255.0, blue: 169.0/255.0, alpha: 1.0)])
            explanationTextView.text = realOption_detail
        }
        
        self.navigationItem.title = segueVC
        
        // Constraints elements
        constraintsExplanationLabel ()
        constraintsSubjectTextField ()
        constraintsExplanationTextView ()
        constraintsButtonSend ()
        
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(self.dismissKeyboardTextField (_:)))
        self.view.addGestureRecognizer(tapGesture)
    }
    
    
    // MARK: - Métodos para el text field
    
    @objc func dismissKeyboardTextField (_ sender: UITapGestureRecognizer) {
        subjectTextField.resignFirstResponder()
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        subjectTextField.resignFirstResponder()
        return true
    }
    
    func textFieldDidBeginEditing(_ textField: UITextField) {
        textField.placeholder = ""
    }
    
    func textFieldDidEndEditing(_ textField: UITextField, reason: UITextField.DidEndEditingReason) {
        if textField.placeholder == "" {
            textField.placeholder = realOption_topic
        }
    }
    

    // MARK: - Métodos para el text view
    
    func textViewDidBeginEditing(_ textView: UITextView) {
        if textView.text == realOption_detail {
            textView.text = ""
            textView.textColor = UIColor(red: 33.0/255.0, green: 33.0/255.0, blue: 33.0/255.0, alpha: 1.0)
        }
    }
    
    func textViewDidEndEditing(_ textView: UITextView) {
        if textView.text.isEmpty {
            textView.text = realOption_detail
            textView.textColor = UIColor(red: 169.0/255.0, green: 169.0/255.0, blue: 169.0/255.0, alpha: 1.0)
        }
    }
    
    func textView(_ textView: UITextView, shouldChangeTextIn range: NSRange, replacementText text: String) -> Bool {
        if text == "\n" {
            textView.resignFirstResponder()
            return false
        }
        return true
    }
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        view.endEditing(true)
        super.touchesBegan(touches, with: event)
    }
    
    
    // MARK: - Constraints y tamaños
    
    func constraintsExplanationLabel () {
        explanationLabel.translatesAutoresizingMaskIntoConstraints = false
        explanationLabel.widthAnchor.constraint(equalToConstant: fullWidthScreen * 0.90).isActive = true
        explanationLabel.centerXAnchor.constraint(equalTo: self.view.centerXAnchor).isActive = true
        explanationLabel.topAnchor.constraint(equalTo: presentationView.bottomAnchor, constant: 15.0).isActive = true
    }
    
    func constraintsSubjectTextField () {
        subjectTextField.translatesAutoresizingMaskIntoConstraints = false
        subjectTextField.widthAnchor.constraint(equalToConstant: fullWidthScreen * 0.90).isActive = true
        subjectTextField.centerXAnchor.constraint(equalTo: self.view.centerXAnchor).isActive = true
        subjectTextField.topAnchor.constraint(equalTo: explanationLabel.bottomAnchor, constant: 15.0).isActive = true
        subjectTextField.layer.cornerRadius = 10
    }
    
    func constraintsExplanationTextView () {
        explanationTextView.translatesAutoresizingMaskIntoConstraints = false
        explanationTextView.widthAnchor.constraint(equalToConstant: fullWidthScreen * 0.90).isActive = true
        explanationTextView.centerXAnchor.constraint(equalTo: self.view.centerXAnchor).isActive = true
        explanationTextView.topAnchor.constraint(equalTo: subjectTextField.bottomAnchor, constant: 12.0).isActive = true
        explanationTextView.bottomAnchor.constraint(equalTo: sendView.topAnchor, constant: -12.0).isActive = true
        explanationTextView.backgroundColor = UIColor.white
        explanationTextView.layer.cornerRadius = 10
    }
    
    
    
    func constraintsButtonSend () {
        
        sendView.translatesAutoresizingMaskIntoConstraints = false
        sendButton.translatesAutoresizingMaskIntoConstraints = false
        sendImage.translatesAutoresizingMaskIntoConstraints = false
        
        // Constraints de la VIEW
        sendView.widthAnchor.constraint(equalToConstant: fullWidthScreen * 0.90).isActive = true
        sendView.centerXAnchor.constraint(equalTo: self.view.centerXAnchor).isActive = true
        sendView.heightAnchor.constraint(equalToConstant: (fullHeightScreen - 174) * 0.10).isActive = true
        sendView.bottomAnchor.constraint(equalTo: self.view.bottomAnchor, constant: -(fullHeightScreen - 174)*0.05).isActive = true
        
        // Constraints del BUTTON
        sendButton.topAnchor.constraint(equalTo: sendView.topAnchor).isActive = true
        sendButton.bottomAnchor.constraint(equalTo: sendView.bottomAnchor).isActive = true
        sendButton.leftAnchor.constraint(equalTo: sendView.leftAnchor).isActive = true
        sendButton.rightAnchor.constraint(equalTo: sendView.rightAnchor).isActive = true
        
        // Constraints de la IMAGE
        sendImage.heightAnchor.constraint(equalToConstant: sendView.frame.size.height * 0.45).isActive = true
        sendImage.centerYAnchor.constraint(equalTo: sendView.centerYAnchor).isActive = true
        sendImage.widthAnchor.constraint(equalToConstant: (sendView.frame.size.height * 0.45)*2).isActive = true
        sendImage.centerXAnchor.constraint(equalTo: sendView.centerXAnchor).isActive = true
        
        // Diseño de los elementos
        sendView.layer.cornerRadius = 12
        sendView.layer.borderWidth = 0.25
        sendView.layer.borderColor = UIColor.white.cgColor
        sendView.backgroundColor = secondaryBlueColor
        
    }
    
    
    
    func InitialAndEndPoint () {
        // height of common elements: statusBar, navBar and tabBar
        statusBarHeight = getStatusBarHeight()
        navBarHeight = getNavBarHeight(navigationController: self.navigationController!)
        // real height and real width of screen
        realHeightScreen = fullHeightScreen - 174
        realWidthScreen = fullWidthScreen
        // initial point and end point of screen
        initialPoint = statusBarHeight + navBarHeight + presentationView.frame.size.height
        endPoint = fullHeightScreen
    }
    
    
    // MARK: - Envío del correo electrónico
    
    @IBAction func sendButton(_ sender: UIButton) {
        let mailComposeViewController = configuredMailComposeViewController()
        if MFMailComposeViewController.canSendMail() { self.present(mailComposeViewController, animated: true) }
        else { self.showSendMailErrorAlert() }
    }
    
    func configuredMailComposeViewController() -> MFMailComposeViewController {
        let mailComposeVC = MFMailComposeViewController ()
        mailComposeVC.mailComposeDelegate = self
        mailComposeVC.setToRecipients(["app_tituloper@hotmail.com"])
        mailComposeVC.setSubject(subjectTextField.text!)
        if (explanationTextView.text == realOption_detail) {
            mailComposeVC.setMessageBody("", isHTML: false)
        } else {
            mailComposeVC.setMessageBody(explanationTextView.text, isHTML: false)
        }
        
        return mailComposeVC
    }
    
    func showSendMailErrorAlert () {
        let alert = UIAlertController(title: "No se pudo enviar el mensaje", message: "Por favor, revisa la configuración de tu e-mail y vuelve a intentarlo", preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: "OK", style: .default, handler: nil))
        self.present(alert, animated: true, completion: nil)
    }
    
    func mailComposeController(_ controller: MFMailComposeViewController, didFinishWith result: MFMailComposeResult, error: Error?) {
        
        switch result {
            case MFMailComposeResult.cancelled:
                self.dismiss(animated: true, completion: nil)
            case MFMailComposeResult.failed:
                self.showSendMailErrorAlert()
                self.dismiss(animated: true, completion: nil)
            case MFMailComposeResult.sent:
                self.dismiss(animated: true, completion: nil)
                let alert : UIAlertController = UIAlertController(title: "Mensaje enviado correctamente", message: "¡Gracias por escribirnos!", preferredStyle: .alert)
                let actionOk : UIAlertAction = UIAlertAction(title: "Ok", style: .default, handler: nil)
                alert.addAction(actionOk)
                present(alert, animated: true)
                subjectTextField.text = ""
                explanationTextView.text = realOption_detail
                explanationTextView.textColor = UIColor(red: 169.0/255.0, green: 169.0/255.0, blue: 169.0/255.0, alpha: 1.0)
            default:
                break
        }
        
    }
    
    
    
    
    

}
