//
//  SideMenuViewController.swift
//  PER
//
//  Created by Conrado on 16/04/2020.
//  Copyright © 2020 Conrado. All rights reserved.
//

import UIKit

class SideMenuViewController: UIViewController, UITableViewDelegate, UITableViewDataSource {
    
    // Imagen de portada
    @IBOutlet weak var viewImageBG: UIView!
    @IBOutlet weak var imageBG: UIImageView!
    @IBOutlet weak var proverbBG: UILabel!
    
    // TableView
    @IBOutlet weak var tableView: UITableView!
    
    var optionsSelectedMenu : [String]!
    let section0 : [String] = ["Inicio"]
    let section1 : [String] = ["Temario oficial"]
    let section2 : [String] = ["Incidencias", "Sugerencias"]
    let section3 : [String] = ["Comparte la app"]
    let section4 : [String] = ["Puntúame en la App Store"]
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        tableView.delegate = self
        tableView.dataSource = self
        
        tableView.backgroundColor = secondaryBlueColor
        
        constraintsInitialView ()
        
    }
    
    func constraintsInitialView () {
        
        viewImageBG.frame = CGRect(x: 0, y: 0, width: fullWidthScreen * 0.85, height: fullHeightScreen * 0.275)
        
        imageBG.translatesAutoresizingMaskIntoConstraints = false
        imageBG.leadingAnchor.constraint(equalTo: viewImageBG.leadingAnchor).isActive = true
        imageBG.rightAnchor.constraint(equalTo: viewImageBG.rightAnchor).isActive = true
        imageBG.topAnchor.constraint(equalTo: viewImageBG.topAnchor).isActive = true
        imageBG.bottomAnchor.constraint(equalTo: viewImageBG.bottomAnchor).isActive = true
        
        // Falta por alinear el texto de la etiqueta abajo del todo ...
        proverbBG.translatesAutoresizingMaskIntoConstraints = false
        proverbBG.leadingAnchor.constraint(equalTo: viewImageBG.leadingAnchor, constant: 5).isActive = true
        proverbBG.rightAnchor.constraint(equalTo: viewImageBG.rightAnchor, constant: -5).isActive = true
        proverbBG.bottomAnchor.constraint(equalTo: viewImageBG.bottomAnchor).isActive = true
        proverbBG.heightAnchor.constraint(equalToConstant: viewImageBG.frame.size.height * 0.25).isActive = true

        proverbBG.text = "\"Los vientos y las olas están siempre al lado de los navegantes más expertos\". Edward Gibbon"
        
        tableView.translatesAutoresizingMaskIntoConstraints = false
        tableView.topAnchor.constraint(equalTo: viewImageBG.bottomAnchor).isActive = true
        tableView.leadingAnchor.constraint(equalTo: self.view.leadingAnchor).isActive = true
        tableView.widthAnchor.constraint(equalToConstant: fullWidthScreen * 0.85).isActive = true
        tableView.bottomAnchor.constraint(equalTo: self.view.bottomAnchor).isActive = true
        
    }
    

    // MARK: - Methods TableView
    
    let heightSection : CGFloat = 37.5
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 5
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if ( section == 0 ) {  return section0.count  }
        else if (section == 1) {  return section1.count  }
        else if (section == 2) {  return section2.count  }
        else if (section == 3) {  return section3.count  }
        else {  return section4.count  }
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCell(withIdentifier: "sideMenuCell") as! SideMenuCell
        cell.setConstraints()
        
        cell.backgroundColor = secondaryBlueColor
        
        if (indexPath.section == 0) {
            cell.nameApart.text = section0[indexPath.row]
            cell.iconView.image = UIImage(systemName: "house.fill")
        }
        
        else if (indexPath.section == 1) {
            if (section1[indexPath.row] == "Temario oficial") {
                cell.nameApart.text = section1[indexPath.row]
                cell.iconView.image = UIImage(systemName: "pencil")
            }
        }
        
        else if (indexPath.section == 2) {
            if (section2[indexPath.row] == "Incidencias") {
                cell.nameApart.text = section2[indexPath.row]
                cell.iconView.image = UIImage(systemName: "exclamationmark.circle.fill")
            } else if (section2[indexPath.row] == "Sugerencias") {
                cell.nameApart.text = section2[indexPath.row]
                cell.iconView.image = UIImage(systemName: "plus.bubble.fill")
            }
        }
        
        else if (indexPath.section == 3) {
            cell.nameApart.text = section3[indexPath.row]
            cell.iconView.image = UIImage(systemName: "square.and.arrow.up.on.square.fill")
        }
        
        else {
            cell.nameApart.text = section4[indexPath.row]
            cell.iconView.image = UIImage(systemName: "suit.heart.fill")
        }
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        if section == 0 {
            return 35
        } else {
            return heightSection
        }
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 50
    }

    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        
        let returnedView = UIView(frame: CGRect(x: 0, y: 0, width: self.tableView.frame.size.width, height: heightSection))
        returnedView.backgroundColor = mainBlueColor
        let label = UILabel(frame: CGRect(x: 15, y: 0, width: self.tableView.frame.size.width, height: heightSection))
        label.font = UIFont(name:"IBMPlexSans-LightItalic", size: 15)
        label.textColor = .white
        
        if (section == 0) {  label.text = "Inicio"  }
        else if (section == 1) {  label.text = "Sobre el examen"  }
        else if (section == 2) {  label.text = "Sobre la app"  }
        else if (section == 3) {  label.text = "Redes sociales"  }
        else {  label.text = "Valoración"  }
        
        returnedView.addSubview(label)
        return returnedView
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if (indexPath.section == 0) {
            self.performSegue(withIdentifier: section0[indexPath.row], sender: self)
        } else if (indexPath.section == 1) {
            self.performSegue(withIdentifier: section1[indexPath.row], sender: self)
        } else if (indexPath.section == 2) {
            self.performSegue(withIdentifier: section2[indexPath.row], sender: self)
        } else if (indexPath.section == 3) {
            shareApp ()
        } else {
            scoreTheApp ()
        }
    }
    
    
    // MARK: - Share with other apps
    
    func shareApp () {
        let activityVC = UIActivityViewController(activityItems: ["Para aquellos/as interesados/as en examinarse de la titulación náutica de recreo del PER, ¡nautiPER es la aplicación! \n Échale un vistazo en la App Store y nos lo cuentas ;)"], applicationActivities: nil)
        activityVC.popoverPresentationController?.sourceView = self.view
        self.present(activityVC, animated: true, completion: nil)
    }
    
    // MARK: - Score the app
    func scoreTheApp () {
        
        if let url = URL(string: "https://www.apple.com/es/ios/app-store/") {
            UIApplication.shared.open(url)
        }
    
    }
    
    
    
    // MARK: - Segue
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "Incidencias" || segue.identifier == "Sugerencias" {
            if let nav = segue.destination as? UINavigationController {
                if let vc = nav.visibleViewController as? IncidenceViewController {
                    vc.segueVC = segue.identifier!
                }
            }
        }
    }
    
}
