//
//  VirtualShopViewController.swift
//  PER
//
//  Created by Conrado on 13/04/2020.
//  Copyright © 2020 Conrado. All rights reserved.
//

import UIKit

class VirtualShopViewController : UIViewController, UICollectionViewDelegate, UICollectionViewDataSource, UICollectionViewDelegateFlowLayout {
    
    @IBOutlet weak var menuButton: UIBarButtonItem!
    
    @IBOutlet weak var collectionView: UICollectionView!
    @IBOutlet weak var presentationView: UIView!
    
    // Tamaño de pantalla
    var realHeightScreen : CGFloat!
    var realWidthScreen : CGFloat!
    var initialPoint : CGFloat!
    var endPoint : CGFloat!
    var statusBarHeight : CGFloat!
    var navBarHeight : CGFloat!
    var tabBarHeight : CGFloat!
    
    // Margen del collectionView
    var marginsCollectionView : CGFloat = 0.0
    
    // Tamaño del collectionView
    var widthCollectionView : CGFloat = 0.0
    var heightCollectionView : CGFloat = 0.0
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        collectionView.delegate = self
        collectionView.dataSource = self
        
        self.view.backgroundColor = backgroundColor
        
        // Se asigna el diseño de la barra de navegación
        setDesignNavBar(navigationItem: navigationItem, appearance: UINavigationBarAppearance())
        
        InitialAndEndPoint ()
        
        // Posicionamiento del CollectionView
        marginsCollectionView = realWidthScreen * 0.03
        widthCollectionView = realWidthScreen - (marginsCollectionView * 2)
        heightCollectionView = endPoint - initialPoint - (marginsCollectionView * 2)
        collectionView.frame = CGRect (
            x: marginsCollectionView,
            y: initialPoint + marginsCollectionView,
            width: widthCollectionView,
            height: heightCollectionView
        )
        
        menuButton.target = self.revealViewController()
        menuButton.action = #selector(SWRevealViewController.revealToggle(_:))
        self.view.addGestureRecognizer(self.revealViewController().panGestureRecognizer())
        self.revealViewController().rearViewRevealWidth = fullWidthScreen * 0.85
        
        presentationView.clipsToBounds = true
        presentationView.layer.cornerRadius = 27
        presentationView.layer.maskedCorners = [.layerMaxXMaxYCorner, .layerMinXMaxYCorner]
        
    }
    
    func numberOfSections(in collectionView: UICollectionView) -> Int {
        return 1
    }
    
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return 18
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "bookCell", for: indexPath) as! BookCollectionViewCell
        
        cell.constraintsElements(heightCell: cell.frame.size.height, widthCell: cell.frame.size.width)
        
        cell.layer.cornerRadius = 10
        cell.backgroundColor = backgroundColor
        // cell.layer.borderColor = UIColor.gray.cgColor
        // cell.layer.borderWidth = 1
        
        
        cell.numberBook.text = "\(indexPath.row+1)/18"
        
        return cell
    }
    
    
    // Asignación de altura y anchura de cada celda/item
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        
        let width_cell : CGFloat = collectionView.frame.size.width
        let height_cell : CGFloat = collectionView.frame.size.height
                
        return CGSize(width: width_cell, height: height_cell)
    }
    
    // Asignación de margen izquierdo y derecho entre líneas de Collection View
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, minimumLineSpacingForSectionAt section: Int) -> CGFloat {
        return 10
    }
    
    
    
    
    
    
    func InitialAndEndPoint () {
        // height of common elements: statusBar, navBar and tabBar
        statusBarHeight = getStatusBarHeight()
        navBarHeight = getNavBarHeight(navigationController: self.navigationController!)
        tabBarHeight = getTabBarHeight(tabBarController: self.tabBarController!)
        // real height and real width of screen
        realHeightScreen = fullHeightScreen - statusBarHeight - navBarHeight - tabBarHeight - presentationView.frame.size.height
        realWidthScreen = fullWidthScreen
        // initial point and end point of screen
        initialPoint = getTopBarHeight(statusBarHeight: statusBarHeight, navBarHeight: navBarHeight) + presentationView.frame.size.height
        endPoint = fullHeightScreen - tabBarHeight
    }
    
    
    
    
    
    

}
